#include "OpenWBOSolver.h"

using namespace std;

using namespace NSPACE;

OpenWBOSolver::OpenWBOSolver()
{
	double initial_time = cpuTime();
    solver = new MSU3(_VERBOSITY_MINIMAL_, _INCREMENTAL_ITERATIVE_, _CARD_TOTALIZER_);
    solver->setInitialTime(initial_time);
    solver->setProblemType(_UNWEIGHTED_);
}

void OpenWBOSolver::add_hard_clause(vector<int> & clause)
{
	vec<Lit> lits;
    for (int i = 0; i < clause.size(); i++) {
        int var = abs(clause[i])-1;
        while (var >= solver->nVars()) solver->newVar();
        lits.push((clause[i] > 0) ? mkLit(var) : ~mkLit(var));
    }
    solver->addHardClause(lits);
    hard_clauses.push_back(clause);
}

void OpenWBOSolver::add_soft_clause(int weight, vector<int> & clause)
{
	vec<Lit> lits;
	for (int i = 0; i < clause.size(); i++) {
        int var = abs(clause[i])-1;
        while (var >= solver->nVars()) solver->newVar();
        lits.push((clause[i] > 0) ? mkLit(var) : ~mkLit(var));
    }
    solver->setCurrentWeight(weight);
    solver->updateSumWeights(weight);
    solver->addSoftClause(weight, lits);
    soft_clauses.push_back(make_pair(weight, clause));
}

void OpenWBOSolver::solve()
{
    if (solver->getCurrentWeight() == 1) {
        solver->setProblemType(_UNWEIGHTED_);
        solver->search();
    } else {
        solver->setProblemType(_WEIGHTED_);
        bool bmo_strategy = solver->isBMO(false);
        MaxSAT * mxsolver;
        if (bmo_strategy)
            mxsolver = new WMSU3(_VERBOSITY_MINIMAL_, _INCREMENTAL_ITERATIVE_, _CARD_TOTALIZER_, _PB_SWC_, true);
        else
            mxsolver = new WBO(_VERBOSITY_MINIMAL_, _WEIGHT_DIVERSIFY_, true, 500000);
        solver->copySolver(mxsolver);
        delete solver;
        solver = mxsolver;
        double initial_time = cpuTime();
        solver->setInitialTime(initial_time);
        solver->search();
    }
	for (int i = 0; i < solver->model.size(); i++) {
        assignment[i] = (solver->model[i] == l_True) ? 1 : 0;
    }
    /* delete solver;
    double initial_time = cpuTime();
    solver = new MSU3(_VERBOSITY_MINIMAL_, _INCREMENTAL_ITERATIVE_, _CARD_TOTALIZER_);
    solver->setInitialTime(initial_time);
    solver->setProblemType(_UNWEIGHTED_);
    for (int i = 0; i < hard_clauses.size(); i++) {
        vec<Lit> lits;
        for (int j = 0; j < hard_clauses[i].size(); j++) {
            int var = abs(hard_clauses[i][j])-1;
            while (var >= solver->nVars()) solver->newVar();
            lits.push((hard_clauses[i][j] > 0) ? mkLit(var) : ~mkLit(var));
        }
        solver->addHardClause(lits);
    }
    for (int i = 0; i < soft_clauses.size(); i++) {
        vec<Lit> lits;
        for (int j = 0; j < soft_clauses[i].second.size(); j++) {
            int var = abs(soft_clauses[i].second[j])-1;
            while (var >= solver->nVars()) solver->newVar();
            lits.push((soft_clauses[i].second[j] > 0) ? mkLit(var) : ~mkLit(var));
        }
        solver->addSoftClause(soft_clauses[i].first, lits);
    } */
}